import('lang.Object')
import('model.GuiEvent')
import('model.toolbar.Toolbar')

---@class application.Interface : lang.Object
---@field private _player LuaPlayer
Interface = extends(Object)

---@public
---@return application.Interface
function Interface:new(player_index)
    local this = Interface:super(Object:new())
    this._player = game.players[player_index]
    return this
end

---@public
function Interface:createToolbar()
    Toolbar:create(self._player.gui.screen):centerOnScreen()
end

---@public
---@param eventData EventData
function Interface:handle(eventData)
    local guiEvent = GuiEvent:new(eventData)
    if Interface:isOwnerOf(guiEvent) and Toolbar:contains(guiEvent:element()) then
        Toolbar:findUp(guiEvent:element()):handle(guiEvent)
    end
end

---@private
---@param guiEvent model.GuiEvent
function Interface:isOwnerOf(guiEvent)
    return guiEvent:modName() == "interface-mod"
end

---@public
function Interface:updateToolbars()
    local toolbars = self:toolbars()
    for _, toolbar in ipairs(toolbars) do
        toolbar:update()
    end
end

---@public
function Interface:deleteAll()
    for _, toolbar in ipairs(self:toolbars()) do
        toolbar:delete()
    end
end

---@public
---@return model.toolbar.Toolbar[]
function Interface:toolbars()
    ---@type model.toolbar.Toolbar[]
    local toolbars = {}
    for _, child in ipairs(self._player.gui.screen.children) do
        if Toolbar:is(child) then
            table.insert(toolbars, Toolbar:new(child))
        end
    end
    return toolbars
end
