import('lang.Object')

---@class model.Cursor : lang.Object
---@field private _player LuaPlayer
Cursor = extends(Object)

---@return model.Cursor
---@param player_index number
function Cursor:new(player_index)
    local this = Cursor:super(Object:new())
    this._player = game.players[player_index]
    return this
end

---@public
---@return boolean
function Cursor:holdsItem()
    return self:holdsSomething() and not (self:holdsTool() or self:holdsBlueprint() or self:holdsPlanner())
end

---@public
---@return boolean
function Cursor:holdsTool()
    return self:holdsSomething() and (self:itemType() == "selection-tool" or self:itemType() == "copy-paste-tool")
end

---@public
---@return boolean
function Cursor:holdsBlueprint()
    return self:holdsSomething() and self._player.is_cursor_blueprint()
end

---@public
---@return boolean
function Cursor:holdsPlanner()
    return self:holdsSomething() and (self:itemType() == "deconstruction-item" or self:itemType() == "upgrade-item")
end

---@private
---@return boolean
function Cursor:holdsSomething()
    return not self._player.is_cursor_empty() and (self:holdsStack() or self:holdsGhost())
end

---@private
---@return string
function Cursor:itemType()
    return self:item().type
end

---@public
---@return string
function Cursor:itemName()
    return self:item().name
end

---@private
---@return LuaItemStack | LuaItemPrototype | string
function Cursor:item()
    if self:holdsStack() then
        return self._player.cursor_stack
    elseif self:holdsGhost() then
        return self._player.cursor_ghost
    else
        error("Called for an item when cursor is empty")
    end
end

---@public
---@return boolean
function Cursor:holdsStack()
    return self._player.cursor_stack and self._player.cursor_stack.valid_for_read and self._player.cursor_stack.name
end

---@public
---@return LuaItemStack
function Cursor:stack()
    return self._player.cursor_stack
end

---@private
---@return boolean
function Cursor:holdsGhost()
    return self._player.cursor_ghost and self._player.cursor_ghost.name
end

---@public
---@param stack LuaItemStack
---@return boolean successful
function Cursor:pickStack(stack)
    self:clear()
    return self._player.cursor_stack.transfer_stack(stack)
end

---@public
---@param itemName string
function Cursor:pickGhost(itemName)
    self:clear()
    self._player.cursor_ghost = itemName
end

---@public
---@param toolName string
function Cursor:pickTool(toolName)
    self:clear()
    self._player.cursor_stack.set_stack { name = toolName, count = 1 }
end

---@public
function Cursor:clear()
    self._player.clear_cursor()
end
