import('lang.Object')
import('model.Component')
import('model.toolbar.header.ToolbarHeader')
import('model.toolbar.section.Section')

---@class model.toolbar.Toolbar : model.Component
Toolbar = extends(Component)
Toolbar.__componentType = "toolbar.Toolbar"

function Toolbar:create(parent)
    local root = Toolbar:createRootElement(parent, {
        type = "frame",
        direction = "vertical",
        style = "toolbar" })
    ToolbarHeader:create(root)
    local content = root.add {
        type = "flow",
        name = "content",
        direction = "vertical",
        style = "toolbar_content" }
    Section:create(content)
    return Toolbar:new(root)
end

function Toolbar:new(element)
    return Toolbar:super(Component:new(element))
end

function Toolbar:childrenClasses()
    return { ToolbarHeader, Section }
end

---@public
function Toolbar:centerOnScreen()
    local resolution = game.players[self:root().player_index].display_resolution
    self:root().location = { resolution.width / 2 - 113, resolution.height / 2 }
end

---@public
function Toolbar:addSection()
    Section:create(self:content())
end

---@public
function Toolbar:collapse()
    self:content().visible = false
end

---@public
function Toolbar:expand()
    self:content().visible = true
end

---@public
function Toolbar:update()
    for _, section in ipairs(self:sections()) do
        section:update()
    end
end

function Toolbar:content()
    return self:root().content
end

---@public
---@return model.toolbar.section.Section[]
function Toolbar:sections()
    return Section:findDown(self:root())
end
