import('model.Component')
import('model.toolbar.Toolbar')
import('model.toolbar.section.header.MoveDown')
import('model.toolbar.section.header.MoveUp')
import('model.toolbar.section.header.ExpandSection')
import('model.toolbar.section.header.CollapseSection')
import('model.toolbar.section.header.DeleteSection')
import('model.toolbar.section.header.CancelDeleteSection')
import('model.toolbar.section.header.ConfirmDeleteSection')

---@class model.toolbar.section.header.SectionHeader : model.Component
SectionHeader = extends(Component)
SectionHeader.__componentType = "model.toolbar.section.header.SectionHeader"

function SectionHeader:create(parent)
    local root = SectionHeader:createRootElement(parent, {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_section_header" })
    local toolbar = Toolbar:findUp(parent):root()
    root.drag_target = toolbar

    MoveDown:create(root)
    MoveUp:create(root)
    local name = root.add { type = "textfield", style = "toolbar_section_header_name" }
    name.lose_focus_on_confirm = true
    CollapseSection:create(root)
    DeleteSection:create(root)

    return SectionHeader:new(root)
end

function SectionHeader:new(element)
    return SectionHeader:super(Component:new(element))
end

function SectionHeader:childrenClasses()
    return {
        MoveDown, MoveUp,
        CollapseSection, ExpandSection,
        DeleteSection, CancelDeleteSection, ConfirmDeleteSection }
end

---@public
function SectionHeader:askForDeletion()
    DeleteSection:findDownFirst(self:root()):delete()
    ConfirmDeleteSection:create(self:root())
    CancelDeleteSection:create(self:root())
end

---@public
function SectionHeader:cancelDeletion()
    CancelDeleteSection:findDownFirst(self:root()):delete()
    ConfirmDeleteSection:findDownFirst(self:root()):delete()
    DeleteSection:create(self:root())
end

function SectionHeader:lock()
    if #Toolbar:findUp(self:root()):sections() == 1 then
        self:hide()
    end
    Component.lock(self)
end

function SectionHeader:unlock()
    self:show()
    Component.unlock(self)
end
