import('model.Component')
import('model.toolbar.section.table.Table')
import('model.toolbar.section.table.slots.Empty')
import('model.toolbar.section.table.slots.Item')

---@class model.toolbar.section.table.Row : model.Component
Row = extends(Component)

function Row:create(parent)
    local root = parent.add {
        type = "flow",
        direction = "horizontal",
        style = "toolbar_section_table_content_row" }

    local table = Table:findUp(parent)
    for _ = 1, table:columnsCount() ~= 0 and table:columnsCount() or 5 do
        Empty:create(root)
    end
    return Row:new(root)
end

---@public
---@return model.toolbar.section.table.Row
---@param element LuaGuiElement
function Row:new(element)
    return Row:super(Component:new(element))
end

function Row:addColumn()
    Empty:create(self:root())
end

---@return model.toolbar.section.table.slots.Item[]
function Row:items()
    local items = {}
    for _, element in ipairs(self:root().children) do
        if Item:is(element) then
            table.insert(items, Item:new(element))
        end
    end
    return items
end

---@return boolean
function Row:hasNonEmptySlotInLastColumn()
    return not Empty:is(self:lastColumn())
end

function Row:deleteLastColumn()
    self:lastColumn().destroy()
end

---@return LuaGuiElement
function Row:lastColumn()
    return self:root().children[#self:root().children]
end

---@return boolean
function Row:hasNonEmptySlots()
    for _, slot in ipairs(self:root().children) do
        if not Empty:is(slot) then
            return true
        end
    end
    return false
end
