import('lang.Object')
import('model.Component')
import('model.Cursor')
import('model.toolbar.section.table.History')
import('model.toolbar.section.table.Inventory')

---@class model.toolbar.section.table.Slot : model.Component
---@field private _player_index number
---@field private _history model.toolbar.section.table.History
Slot = extends(Component)

---@protected
---@return model.toolbar.section.table.Slot
---@param element LuaGuiElement
function Slot:new(element)
    local this = Slot:super(Component:new(element))
    this._player_index = this:root().player_index
    this._history = History:getInstanceFor(this._player_index)
    return this
end

---@protected
---@param itemName string
---@return boolean
function Slot:wasDragged(itemName)
    return self._history:hasPickedSlot() and self._history:pickedSlot():isValid()
            and self._history:pickedSlot():itemName() == itemName
end

---@protected
function Slot:drag()
    self._history:pickedSlot():clear()
end

---@protected
function Slot:rememberPick()
    self._history:rememberPick(self)
end

---@protected
function Slot:forgetPick()
    self._history:forgetPick()
end

---@protected
function Slot:clear()
    self:replaceWith(Empty:create(self:parent()))
end

---@protected
---@return model.Cursor
function Slot:cursor()
    return Cursor:new(self._player_index)
end

---@protected
---@return model.toolbar.section.table.Inventory
function Slot:inventory()
    return Inventory:new(self._player_index)
end

---@private
---@return number
function Slot:index()
    return self:root().get_index_in_parent()
end

---@protected
---@return boolean
function Slot:hasItem()
    return self:itemName() ~= nil
end

---@protected
---@return string
function Slot:itemName()
    error("Not implemented")
end
