import('model.Component')
import('model.toolbar.section.table.header.TableHeader')
import('model.toolbar.section.table.Row')
import('model.toolbar.section.table.slots.Empty')

---@class model.toolbar.section.table.Table : model.Component
Table = extends(Component)
Table.__componentType = "model.toolbar.section.table.Table"

function Table:create(parent)
    local toolbar = Toolbar:findUp(parent):root()
    local root = Table:createRootElement(parent, {type = "frame", direction = "vertical", style = "toolbar_section_table"})
    root.drag_target = toolbar
    TableHeader:create(root)
    local frame = root.add {
        type = "frame",
        name = "frame",
        direction = "vertical",
        style = "toolbar_section_table"
    }
    frame.drag_target = toolbar
    local content = frame.add {
        type = "flow",
        name = "content",
        direction = "vertical",
        style = "toolbar_section_table_content" }
    content.drag_target = toolbar
    local table = Table:new(root)
    table:addRow()
    return table
end

function Table:new(element)
    return Table:super(Component:new(element))
end

function Table:childrenClasses()
    return {TableHeader, Empty, Item, Tool}
end

function Table:update()
    for _, item in ipairs(self:items()) do
        item:update()
    end
end

---@private
---@return model.toolbar.section.table.slots.Item[]
function Table:items()
    local items = {}
    for _, row in ipairs(self:rows()) do
        for _, item in ipairs(row:items()) do
            table.insert(items, item)
        end
    end
    return items
end

---@public
function Table:addRow()
    Row:create(self:content())
end

---@public
function Table:removeLastRow()
    if self:rowsCount() > 1 then
        local lastRow = Row:new(self:content().children[self:rowsCount()])
        if not lastRow:hasNonEmptySlots() then
            lastRow:delete()
        end
    end
end

---@public
function Table:addColumn()
    for _, row in ipairs(self:rows()) do
        row:addColumn()
    end
end

---@public
function Table:removeLastColumn()
    if self:columnsCount() > 1 and not self:hasNonEmptySlotsInLastColumn() then
        for _, row in ipairs(self:rows()) do
            row:deleteLastColumn()
        end
    end
end

---@private
---@return boolean
function Table:hasNonEmptySlotsInLastColumn()
    for i, row in ipairs(self:rows()) do
        if row:hasNonEmptySlotInLastColumn() then
            return true
        end
    end
    return false
end

---@private
---@return number
function Table:rowsCount()
    return #self:rows()
end

---@private
---@return model.toolbar.section.table.Row[]
function Table:rows()
    local rows = {}
    for _, row in ipairs(self:content().children) do
        table.insert(rows, Row:new(row))
    end
    return rows
end

---@private
---@return number
function Table:columnsCount()
    if self:rowsCount() > 0 then
        return #self:firstRow().children
    else
        return 0
    end
end

---@private
---@return LuaGuiElement
function Table:firstRow()
    return self:content().children[1]
end

---@private
---@return LuaGuiElement
function Table:lastRow()
    return self:content().children[#self:content().children]
end

---@return model.toolbar.section.table.header.TableHeader
function Table:header()
    return TableHeader:findDownFirst(self:root())
end

---@return LuaGuiElement
function Table:content()
    return self:root().frame.content
end
