import('lang.Object')
import('model.toolbar.section.table.Slot')
import('model.toolbar.section.table.slots.Item')
import('model.toolbar.section.table.slots.Tool')

---@class model.toolbar.section.table.slots.Empty : model.toolbar.section.table.Slot
Empty = extends(Slot)
Empty.__componentType = "model.toolbar.section.table.slots.Empty"

function Empty:create(parent)
    local element = Empty:createRootElement(parent, {
        type = "choose-elem-button",
        elem_type = "item",
        style = "toolbar_section_table_content_slot",
        mouse_button_filter = { "left", "middle" } })
    return Empty:new(element)
end

---@param element LuaGuiElement
---@return model.toolbar.section.table.slots.Empty
function Empty:new(element)
    return Empty:super(Slot:new(element))
end

function Empty:handle(guiEvent)
    local cursor = self:cursor()

    if guiEvent:isLeftClick() then
        if cursor:holdsSomething() then
            if cursor:holdsItem() then
                self:setItem(cursor:itemName())
            elseif cursor:holdsTool() then
                -- some items does not trigger chooser change
                self:setTool(cursor:itemName())
            end
            self:handleItemDrag()
        end
    elseif guiEvent:isElementChange() then
        if cursor:holdsBlueprint() or cursor:holdsPlanner() then
            self:root().elem_value = nil
        elseif cursor:holdsItem() then
            self:setItem(cursor:itemName())
            self:handleItemDrag()
        elseif self:hasItem() then
            self:setItem(self:itemName())
        else
            self:setItem(cursor:itemName())
        end
    end
end

function Empty:itemName()
    return self:root().elem_value
end

function Empty:handleItemDrag()
    local cursor = self:cursor()
    if self:wasDragged(self:cursor():itemName()) then
        self:drag()
    end
    cursor:clear()
    self:forgetPick()
end

---@private
---@param itemName string
function Empty:setItem(itemName)
    self:replaceWith(Item:create(self:parent(), itemName))
end

---@private
---@param toolName string
function Empty:setTool(toolName)
    self:replaceWith(Tool:create(self:parent(), toolName))
end
